@@ -27,7 +27,6 @@
 #include "g10lib.h"
 #include "cipher.h"
 #include "bufhelp.h"
-#include "cipher-selftest.h"
 #include "rijndael-internal.h"
 
 #ifdef USE_PADLOCK
@@ -101,4 +100,10 @@ _gcry_aes_padlock_decrypt (const RIJNDAEL_context *ctx,
   return do_padlock(ctx, bx, ax, 1);
 }
 
+void
+_gcry_aes_padlock_prepare_decryption (RIJNDAEL_context *ctx)
+{
+  /* Padlock does not need decryption subkeys. */
+  (void)ctx;
+}
 #endif /* USE_PADLOCK */
